/* Rexx */
/*****************************************************************************\
   WPSInst    - Kleiner Kommandoprozessor zur Installation von WPS-Objekten -
 -----------------------------------------------------------------------------
   PARAMETER
      Datei    Name der akt. Scriptdatei

   RETURN
        0   =  Programm wurde erfolgreich beendet
        1   =  Programmabbruch durch schwerwiegenden Syntaxfehler
 ----------------------------------------------------------------------------- 
   Autor: Jrgen Schwarting         Version: 1.00        Datum: 31.05.1995
\*****************************************************************************/
parse arg Datei

      /* Utility-Funktion laden */
   call RxFuncAdd "SysCreateObject", "RexxUtil", "SysCreateObject"

      /* Abfangen von Syntaxfehler die durch die Funktion   */
      /* SysCreateObject auftreten knnen.                  */
   signal on syntax name SyntaxFehler

      /* ---[ Globale Variablen initialisieren ]----------------------- */
   G.!KomDatei.0 = 0    /* enthlt die gesamte Scriptdatei              */
   G.!Objekt.0   = 0    /* Liste aller zu installierenden Objekte       */
   G.!AktObjekt  = 0    /* enthlt das aktuell zu installierende Objekt */

      /* Script lesen und verarbeiten  */
   if (KommandodateiLesen(Datei) > 0) then
      if (KommandodateiAuswerten(Datei) > 0) then
         call ObjekteErzeugen Datei

exit 0


SyntaxFehler:
/*===========================================================================*\
   Hinweis auf einen aufgetretenden Syntaxfehler ausgeben.
 -----------------------------------------------------------------------------
   G.!AktObjekt   Nummer des zuletzt bearbeiteten Objektes  (Input)
   G.!Objekt.i    Liste aller zu installierenden Objekte    (Input)
\*===========================================================================*/
   i = G.!AktObjekt           
   say copies("C4"x, 55)
   say "Schwerwiegender Fehler im Objekt:" ObjektName(G.!Objekt.i.!Parameter)
   say
   say "    KLASSE    =" G.!Objekt.i.!Klasse
   say "    TITEL     =" G.!Objekt.i.!Titel
   say "    POSITION  =" G.!Objekt.i.!Position
   say "    PARAMETER =" G.!Objekt.i.!Parameter
   say
   say " Die Verarbeitung wird abgebrochen..."
exit 1


ObjekteErzeugen: procedure  expose G.
/*===========================================================================*\
   Hier werden alle zu installierenden Objekt erzeugt.
 -----------------------------------------------------------------------------
   Datei          Name der akt. Scriptdatei
   G.!AktObjekt   Nummer des zuletzt bearbeiteten Objektes  (Output)
   G.!Objekt.i    Liste aller zu installierenden Objekte    (Input )
   RETURN         Anzahl der aufgetretenden Fehler
\*===========================================================================*/
 parse arg Datei
   Fehler = 0        

   say copies("CD"x, 55)
   say substr("STATUS",1,15) substr("MODUS",1,15) "OBJEKT"
   say copies("C4"x, 55)

      /* Objekt fr Objekt wird hier installiert */
   do i = 1 to G.!Objekt.0
      G.!AktObjekt = i        /* Nr. des akt. Objekts merken   */
         /* Objekt erzeugen */
      ok = SysCreateObject( G.!Objekt.i.!Klasse,      ,
                            G.!Objekt.i.!Titel,       ,
                            G.!Objekt.i.!Position,    ,
                            G.!Objekt.i.!Parameter,   ,
                            G.!Objekt.i.!Option             )

         /* Objektname aus der Parameterangabe herausfiltern   */
      ObjName = ObjektName(G.!Objekt.i.!Parameter)

         /* Je nachdem ob das Objekt angelegt werden konnte oder nicht, */
         /* wird ein entsprechender Hinweis ausgegeben.                 */
      if (ok = 1) then
         say substr(" Ok",1,15) substr(G.!Objekt.i.!Option,1,15) ObjName
      else do
         Fehler = Fehler + 1  
         say substr(" Fehler",1,15) substr(G.!Objekt.i.!Option,1,15) ObjName
      end /* else */
   end /* do */
   say copies("CD"x, 55)
return Fehler


ObjektName: procedure  expose G.
/*===========================================================================*\
   Der Objektname wird aus der Parameterangabe herausgefiltert.
 -----------------------------------------------------------------------------
   Parameter      Parameterangabe
   RETURN         Objektname oder "*** UNBEKANNT ***"
\*===========================================================================*/
 arg Parameter          /* in Grobuchstaben */

   if (pos("OBJECTID=", Parameter) > 0) then
      parse value Parameter with . "OBJECTID=" Name ">" .
   else 
      parse value Parameter with . "SHADOWID=" Name ">" .

   if (Name <> '') then
      Name = Name || ">"
   else
      Name = "*** UNBEKANNT ***"
return Name


KommandodateiLesen: procedure  expose G.
/*===========================================================================*\
   Einlesen der Scriptdatei.
 -----------------------------------------------------------------------------
   Datei          Name der akt. Scriptdatei
   G.!KomObjekt   Gesamte Scriptdatei                    (Output)
   RETURN         Anzahl der eingelesenen Zeilen
\*===========================================================================*/
 parse arg Datei
   ok = stream(Datei, 'c', 'open read')
      /* falls erfolgreich geffnet wurde, alle Zeilen einlesen */
   if (ok = 'READY:') then do
      do i = 1 by 1 while (lines(Datei) > 0)
         G.!KomDatei.i = linein(Datei)
      end /* do */
      G.!KomDatei.0 = i - 1
      ok = stream(Datei, 'c', 'close')
   end /* if */
   else
      say "FEHLER:" Datei "kann nicht bearbeitet werden !"
return G.!KomDatei.0


KommandodateiAuswerten: procedure  expose G.
/*===========================================================================*\
   Kommandodatei analysieren und WPS-Objekte herausfiltern.
 -----------------------------------------------------------------------------
   Datei          Name der akt. Scriptdatei
   G.!Objekt.i    Liste aller zu installierenden Objekte    (Output)
   RETURN         Anzahl der ermittelten Objekte
\*===========================================================================*/
 parse arg Datei

   say
   say "WPSInst      - Installation von WPS-Objekten -"
   say

      /* Definition aller bekannten Schlsselworte */
   Schluesselworte = "KLASSE TITEL POSITION PARAMETER OPTION"

   x = 0    /* Index des jeweiligen Objekts */

      /* Die gesamte Scriptdatei wird nach bekannten Schlsselworten    */
      /* durchsucht.                                                    */
   do i = 1 to G.!KomDatei.0
         /* Zeile zerlegen */
      parse var G.!KomDatei.i Wort1 "=" Benutzereintrag
         /* Das erste Wort in Grobuchstaben umwandeln und Leerzeichen  */
         /* entfernen.                                                  */
      Wort1 = strip(translate(Wort1))
         /* Prfen ob es sich um ein Schlsselwort handelt.             */
      if (wordpos(Wort1, Schluesselworte) > 0) then do
         Benutzereintrag = strip(Benutzereintrag)
         if (Benutzereintrag <> '') then do
               /* Wenn als Schlsselwort 'KLASSE' erscheint, mu ein    */
               /* neues Objekt angelegt und initialisiert werden.       */
            if (Wort1 = 'KLASSE') then
               x = InitObjekt(x, Benutzereintrag)
            else
                  /* Alle weiteren Benutzerinformationen werden dem je- */
                  /* weiligen Objektattribut zugeordnet.                */
               interpret "G.!Objekt.x.!"Wort1 "= Benutzereintrag"
         end /* if */
      end /* if */
      else 
         if (i > 1) & (Wort1 <> '') & (left(Wort1,1) <> ';') then do
            say "ACHTUNG: Unbekanntes Schlsselwort in Zeile" i
            say "     ==>" Wort1 "<=="
         end /* if */
   end /* do */
   if (G.!Objekt.0 = 0) then
      say "ACHTUNG: In" Datei "wurden keine Objekte definiert !"
return G.!Objekt.0


InitObjekt: procedure  expose G.
/*===========================================================================*\
   Ein neues Objekt wird angelegt und initialisiert.
 -----------------------------------------------------------------------------
   Nr             Die Objektnummer des vorherigen Objekts
   Klasse         Die gewnschte WPS-Klasse  (z.B. WPFolder)
   G.!Objekt.i    Liste aller zu installierenden Objekte       (Output)
   RETURN         Die akt. Objektnummer
\*===========================================================================*/
 parse arg Nr, Klasse
   i = Nr + 1
   G.!Objekt.i.!Klasse     = Klasse
   G.!Objekt.i.!Titel      = ''
   G.!Objekt.i.!Position   = ''
   G.!Objekt.i.!Parameter  = ''
   G.!Objekt.i.!Option     = 'fail'
   G.!Objekt.0             = i
return i
